/**
 * @license
 * Copyright (c) 2015 The Polymer Project Authors. All rights reserved.
 * This code may only be used under the BSD style license found at
 * http://polymer.github.io/LICENSE.txt
 * The complete set of authors may be found at
 * http://polymer.github.io/AUTHORS.txt
 * The complete set of contributors may be found at
 * http://polymer.github.io/CONTRIBUTORS.txt
 * Code distributed by Google as part of the polymer project is also
 * subject to an additional IP rights grant found at
 * http://polymer.github.io/PATENTS.txt
 */
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ast_value_1 = require("../javascript/ast-value");
const esutil = require("../javascript/esutil");
const jsdoc = require("../javascript/jsdoc");
const js_utils_1 = require("./js-utils");
const polymer_element_mixin_1 = require("./polymer-element-mixin");
const polymer2_config_1 = require("./polymer2-config");
class Polymer2MixinScanner {
    scan(document, visit) {
        return __awaiter(this, void 0, void 0, function* () {
            const visitor = new MixinVisitor(document);
            yield visit(visitor);
            return visitor._mixins;
        });
    }
}
exports.Polymer2MixinScanner = Polymer2MixinScanner;
class MixinVisitor {
    constructor(document) {
        this._mixins = [];
        this._currentMixin = null;
        this._currentMixinNode = null;
        this._currentMixinFunction = null;
        // TODO(rictic): do something with these warnings.
        this._warnings = [];
        this._document = document;
    }
    enterAssignmentExpression(node, parent) {
        if (parent.type !== 'ExpressionStatement') {
            return;
        }
        const parentComments = esutil.getAttachedComment(parent) || '';
        const parentJsDocs = jsdoc.parseJsdoc(parentComments);
        if (this._hasPolymerMixinDocTag(parentJsDocs)) {
            const name = ast_value_1.getIdentifierName(node.left);
            const namespacedName = name ? ast_value_1.getNamespacedIdentifier(name, parentJsDocs) : undefined;
            const sourceRange = this._document.sourceRangeForNode(node);
            const summaryTag = jsdoc.getTag(parentJsDocs, 'summary');
            if (namespacedName) {
                this._currentMixin = new polymer_element_mixin_1.ScannedPolymerElementMixin({
                    name: namespacedName,
                    sourceRange,
                    description: parentJsDocs.description,
                    summary: (summaryTag && summaryTag.description) || '',
                    privacy: js_utils_1.getOrInferPrivacy(namespacedName, parentJsDocs, false),
                    jsdoc: parentJsDocs,
                    mixins: jsdoc.getMixins(this._document, node, parentJsDocs, this._warnings),
                });
                this._currentMixinNode = node;
                this._mixins.push(this._currentMixin);
            }
            else {
                // TODO(rictic): warn for a mixin whose name we can't determine.
            }
        }
    }
    enterFunctionDeclaration(node, _parent) {
        const nodeComments = esutil.getAttachedComment(node) || '';
        const nodeJsDocs = jsdoc.parseJsdoc(nodeComments);
        if (this._hasPolymerMixinDocTag(nodeJsDocs)) {
            const name = node.id.name;
            const namespacedName = name ? ast_value_1.getNamespacedIdentifier(name, nodeJsDocs) : undefined;
            const sourceRange = this._document.sourceRangeForNode(node);
            this._currentMixinFunction = node;
            const summaryTag = jsdoc.getTag(nodeJsDocs, 'summary');
            if (namespacedName) {
                this._currentMixin = new polymer_element_mixin_1.ScannedPolymerElementMixin({
                    name: namespacedName,
                    sourceRange,
                    description: nodeJsDocs.description,
                    summary: (summaryTag && summaryTag.description) || '',
                    privacy: js_utils_1.getOrInferPrivacy(namespacedName, nodeJsDocs, false),
                    jsdoc: nodeJsDocs,
                    mixins: jsdoc.getMixins(this._document, node, nodeJsDocs, this._warnings)
                });
                this._currentMixinNode = node;
                this._mixins.push(this._currentMixin);
            }
            else {
                // Warn about a mixin whose name we can't infer.
            }
        }
    }
    leaveFunctionDeclaration(node, _parent) {
        if (this._currentMixinNode === node) {
            this._currentMixin = null;
            this._currentMixinNode = null;
            this._currentMixinFunction = null;
        }
    }
    enterVariableDeclaration(node, _parent) {
        const comment = esutil.getAttachedComment(node) || '';
        const docs = jsdoc.parseJsdoc(comment);
        const isMixin = this._hasPolymerMixinDocTag(docs);
        const sourceRange = this._document.sourceRangeForNode(node);
        const summaryTag = jsdoc.getTag(docs, 'summary');
        if (isMixin) {
            let mixin = undefined;
            if (node.declarations.length === 1) {
                const declaration = node.declarations[0];
                const name = ast_value_1.getIdentifierName(declaration.id);
                if (name) {
                    const namespacedName = ast_value_1.getNamespacedIdentifier(name, docs);
                    mixin = new polymer_element_mixin_1.ScannedPolymerElementMixin({
                        name: namespacedName,
                        sourceRange,
                        description: docs.description,
                        summary: (summaryTag && summaryTag.description) || '',
                        privacy: js_utils_1.getOrInferPrivacy(namespacedName, docs, false),
                        jsdoc: docs,
                        mixins: jsdoc.getMixins(this._document, declaration, docs, this._warnings)
                    });
                }
            }
            if (mixin) {
                this._currentMixin = mixin;
                this._currentMixinNode = node;
                this._mixins.push(this._currentMixin);
            }
            else {
                // TODO(rictic); warn about being unable to determine mixin name.
            }
        }
    }
    leaveVariableDeclaration(node, _parent) {
        if (this._currentMixinNode === node) {
            this._currentMixin = null;
            this._currentMixinNode = null;
            this._currentMixinFunction = null;
        }
    }
    enterFunctionExpression(node, _parent) {
        if (this._currentMixin != null && this._currentMixinFunction == null) {
            this._currentMixinFunction = node;
        }
    }
    enterArrowFunctionExpression(node, _parent) {
        if (this._currentMixin != null && this._currentMixinFunction == null) {
            this._currentMixinFunction = node;
        }
    }
    enterClassExpression(node, parent) {
        if (parent.type !== 'ReturnStatement' &&
            parent.type !== 'ArrowFunctionExpression') {
            return;
        }
        this._handleClass(node);
    }
    enterClassDeclaration(node, _parent) {
        const comment = esutil.getAttachedComment(node) || '';
        const docs = jsdoc.parseJsdoc(comment);
        const isMixinClass = this._hasPolymerMixinClassDocTag(docs);
        if (isMixinClass) {
            this._handleClass(node);
        }
    }
    _handleClass(node) {
        const mixin = this._currentMixin;
        if (mixin == null) {
            return;
        }
        polymer2_config_1.getProperties(node, this._document).forEach((p) => mixin.addProperty(p));
        polymer2_config_1.getMethods(node, this._document).forEach((m) => mixin.addMethod(m));
        mixin.events = esutil.getEventComments(node);
        // mixin.sourceRange = this._document.sourceRangeForNode(node);
        return mixin;
    }
    _hasPolymerMixinDocTag(docs) {
        const elementTags = docs.tags &&
            docs.tags.filter((t) => t.tag === 'polymerMixin');
        return elementTags && elementTags.length >= 1;
    }
    _hasPolymerMixinClassDocTag(docs) {
        const elementTags = docs.tags &&
            docs.tags.filter((t) => t.tag === 'polymerMixinClass');
        return elementTags && elementTags.length >= 1;
    }
}

//# sourceMappingURL=polymer2-mixin-scanner.js.map
